@@ -7,7 +7,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2015	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -48,22 +48,18 @@ typedef struct WDIPCCmdResult
 extern void wd_ipc_initialize_data(void);
 extern char* get_watchdog_ipc_address(void);
 extern unsigned int* get_ipc_shared_key(void);
-
-
-extern int wd_set_node_mask_for_failback_req(int *node_id_set, int count);
-extern int wd_set_node_mask_for_degenerate_req(int *node_id_set, int count);
-extern int wd_set_node_mask_for_promote_req(int *node_id_set, int count);
-extern int wd_chk_node_mask_for_failback_req(int *node_id_set, int count);
-extern int wd_chk_node_mask_for_degenerate_req(int *node_id_set, int count);
-extern int wd_chk_node_mask_for_promote_req(int *node_id_set, int count);
-
+extern void set_watchdog_process_needs_cleanup(void);
+extern void reset_watchdog_process_needs_cleanup(void);
+extern bool get_watchdog_process_needs_cleanup(void);
+extern void set_watchdog_node_escalated(void);
+extern void reset_watchdog_node_escalated(void);
+extern bool get_watchdog_node_escalation_state(void);
 
 extern WdCommandResult wd_start_recovery(void);
 extern WdCommandResult wd_end_recovery(void);
-extern WdCommandResult wd_send_failback_request(int node_id);
-extern WdCommandResult wd_degenerate_backend_set(int *node_id_set, int count);
-extern WdCommandResult wd_promote_backend(int node_id);
-extern WDFailoverCMDResults wd_send_failover_sync_command(WDFailoverCMDTypes cmdType, char* syncReqType);
+extern WDFailoverCMDResults wd_send_failback_request(int node_id, unsigned int *wd_failover_id);
+extern WDFailoverCMDResults wd_degenerate_backend_set(int *node_id_set, int count, unsigned int *wd_failover_id);
+extern WDFailoverCMDResults wd_promote_backend(int node_id, unsigned int *wd_failover_id);
 
 extern WDPGBackendStatus* get_pg_backend_status_from_master_wd_node(void);
 
@@ -72,13 +68,12 @@ extern char* wd_get_watchdog_nodes(int nodeID);
 extern WDIPCCmdResult* issue_command_to_watchdog(char type, int timeout_sec, char* data, int data_len, bool blocking);
 
 
-/* wd_interlock.c */
-
-extern WDFailoverCMDResults wd_release_failover_command_lock(WDFailoverCMDTypes cmdType);
-extern WDFailoverCMDResults wd_failover_command_check_lock(WDFailoverCMDTypes cmdType);
-extern WDFailoverCMDResults wd_failover_command_end(WDFailoverCMDTypes cmdType);
-extern WDFailoverCMDResults wd_failover_command_start(WDFailoverCMDTypes cmdType);
-extern void wd_wati_until_lock_or_timeout(WDFailoverCMDTypes cmdType);
+/* functions for failover commands interlocking */
+extern WDFailoverCMDResults wd_end_failover_interlocking(unsigned int wd_failover_id);
+extern WDFailoverCMDResults wd_start_failover_interlocking(unsigned int wd_failover_id);
+extern WDFailoverCMDResults wd_failover_lock_release(enum WDFailoverLocks lock, unsigned int wd_failover_id);
+extern WDFailoverCMDResults wd_failover_lock_status(enum WDFailoverLocks lock, unsigned int wd_failover_id);
+extern void wd_wait_until_command_complete_or_timeout(enum WDFailoverLocks lock, unsigned int wd_failover_id);
 
 
 
