@@ -701,6 +701,7 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 			pool_set_skip_reading_from_backends();
 			pool_stats_count_up_num_cache_hits();
 			pool_unset_query_in_progress();
+			pool_unset_pending_response();
 			return POOL_CONTINUE;
 		}
 	}
@@ -787,6 +788,31 @@ POOL_STATUS Execute(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 #ifdef NOT_USED
 		pool_unset_query_in_progress();
 #endif
+
+		/*
+		 * Take of "writing transaction" flag.
+		 */
+		if (!is_select_query(node, query))
+		{
+			/*
+			 * If the query was not READ SELECT, and we are in an
+			 * explicit transaction, remember that we had a write
+			 * query in this transaction.
+			 */
+			if (TSTATE(backend, MASTER_SLAVE ? PRIMARY_NODE_ID : REAL_MASTER_NODE_ID) == 'T')
+			{
+				/* However, if the query is "SET TRANSACTION READ ONLY" or its variant,
+				 * don't set it.
+				 */
+				if (!pool_is_transaction_read_only(node))
+				{
+					ereport(DEBUG1,
+							(errmsg("not SET TRANSACTION READ ONLY")));
+
+					pool_set_writing_transaction();
+				}
+			}
+		}
 	}
 
 	return POOL_CONTINUE;
@@ -1403,6 +1429,18 @@ POOL_STATUS Close(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 		pmsg = pool_pending_messages_create('C', len, contents);
 		pool_pending_message_add(pmsg);
 		pool_unset_query_in_progress();
+		/*
+		 * Remeber that we send flush or sync message to backend.
+		 */
+		pool_unset_pending_response();
+
+		/*
+		 * Remove send message
+		 */
+		ereport(DEBUG1,
+				(errmsg("Close: removing sent message %c %s", *contents, contents+1)));
+
+		pool_remove_sent_message(*contents == 'S'?'P':'B', contents+1);
 	}
 
 	return POOL_CONTINUE;
@@ -2306,7 +2344,6 @@ POOL_STATUS ProcessFrontendResponse(POOL_CONNECTION *frontend,
 			if (!pool_is_query_in_progress() && !pool_is_ignore_till_sync())
 				pool_set_query_in_progress();
 			status = Close(frontend, backend, len, contents);
-			pool_set_pending_response();
 			break;
 
 		case 'D':	/* Describe */
