@@ -3,7 +3,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2017	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -75,10 +75,9 @@ static int pool_get_dml_table_oid(int **oid);
 static int pool_get_dropdb_table_oids(int **oids, int dboid);
 static void pool_discard_dml_table_oid(void);
 static void pool_invalidate_query_cache(int num_table_oids, int *table_oid, bool unlink, int dboid);
-
 static int pool_get_database_oid(void);
 static void pool_add_table_oid_map(POOL_CACHEKEY *cachkey, int num_table_oids, int *table_oids);
-static void pool_reset_memqcache_buffer(bool reset_dml_oids);
+static void pool_reset_memqcache_buffer(void);
 static POOL_CACHEID *pool_add_item_shmem_cache(POOL_QUERY_HASH *query_hash, char *data, int size);
 static POOL_CACHEID *pool_find_item_on_shmem_cache(POOL_QUERY_HASH *query_hash);
 static char *pool_get_item_shmem_cache(POOL_QUERY_HASH *query_hash, int *size, int *sts);
@@ -121,7 +120,7 @@ static uint32 create_hash_key(POOL_QUERY_HASH *key);
 static volatile POOL_HASH_ELEMENT *get_new_hash_element(void);
 static void put_back_hash_element(volatile POOL_HASH_ELEMENT *element);
 static bool is_free_hash_element(void);
-static char *get_relation_without_alias(RangeVar *relation);
+static void forward_pending_data(POOL_CONNECTION *frontend,POOL_CONNECTION *backend);
 
 /*
  * Connect to Memcached
@@ -593,11 +592,7 @@ POOL_STATUS pool_fetch_from_memory_cache(POOL_CONNECTION *frontend,
 	char *qcache;
 	size_t qcachelen;
 	int sts;
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	*foundp = false;
     
@@ -619,53 +614,106 @@ POOL_STATUS pool_fetch_from_memory_cache(POOL_CONNECTION *frontend,
 	pool_shmem_unlock();
 	POOL_SETMASK(&oldmask);
 
-	if (sts == 0)
+	if (sts != 0)
+		/* Cache not found */
+		return POOL_CONTINUE;
+
+	/*
+	 * Cache found. If we are doing extended query and in streaming
+	 * replication mode, we need to retrieve any responses from backend and
+	 * forward them to frontend.
+	 */
+	if (pool_is_doing_extended_query_message() && STREAM)
 	{
-		/*
-		 * Cache found. send each messages to frontend
-		 */
-		send_cached_messages(frontend, qcache, qcachelen);
-		pfree(qcache);
-
-		/*
-		 * Send a "READY FOR QUERY" if not in extended query.
-		 */
-		if (!pool_is_doing_extended_query_message() && MAJOR(backend) == PROTO_MAJOR_V3)
-		{
-			signed char state;
-
-			/*
-			 * We keep previous transaction state.
-			 */
-			state = MASTER(backend)->tstate;
-			send_message(frontend, 'Z', 5, (char *)&state);
-		}
-
-		if (pool_flush(frontend))
-		{
-			return POOL_END;
-		}
-
-		*foundp = true;
-
-		if (pool_config->log_per_node_statement)
-			ereport(LOG,
-				(errmsg("fetch from memory cache"),
-					errdetail("query result fetched from cache. statement: %s", contents)));
+		POOL_SESSION_CONTEXT *session_context;
+		POOL_CONNECTION *target_backend;
 
 		ereport(DEBUG1,
-			(errmsg("fetch from memory cache"),
-				 errdetail("query result found in the query cache, %s", contents)));
+				(errmsg("memcache: fetching forwarding_pending_data")));
 
-		return POOL_CONTINUE;
+		session_context = pool_get_session_context(true);
+		target_backend = CONNECTION(backend, session_context->load_balance_node_id);
+		forward_pending_data(frontend, target_backend);
 	}
 
-	/* Cache not found */
+	/*
+	 * Send each messages to frontend
+	 */
+	send_cached_messages(frontend, qcache, qcachelen);
+	pfree(qcache);
+
+	/*
+	 * If we are doing extended query, forward sync message from frontend to
+	 * backend. This is necessary to prevent receiving Sync message after
+	 * Sending Ready for query.
+	 */
+	if (pool_is_doing_extended_query_message())
+	{
+		char kind;
+		int32 len;
+		POOL_SESSION_CONTEXT *session_context;
+		POOL_CONNECTION *target_backend;
+		char buf[5];
+
+		if (pool_flush(frontend))
+			return POOL_END;
+		if (pool_read(frontend, &kind, 1))
+			return POOL_END;
+
+		ereport(DEBUG2,
+				(errmsg("memcache: fetching from memory cache: expecting sync: kind '%c'", kind)));
+		if (pool_read(frontend, &len, sizeof(len)))
+			return POOL_END;
+
+		/* Forward "Sync" message to backend */
+		session_context = pool_get_session_context(true);
+		target_backend = CONNECTION(backend, session_context->load_balance_node_id);
+		pool_write(target_backend, &kind, 1);
+		pool_write_and_flush(target_backend, &len, sizeof(len));
+
+		/* Read and discard "Ready for query" message from backend */
+		pool_read(target_backend, &kind, 1);
+		pool_read(target_backend, buf, sizeof(buf));
+	}
+
+	/*
+	 * send a "READY FOR QUERY"
+	 */
+	if (MAJOR(backend) == PROTO_MAJOR_V3)
+	{
+		signed char state;
+
+		/*
+		 * We keep previous transaction state.
+		 */
+		state = MASTER(backend)->tstate;
+		send_message(frontend, 'Z', 5, (char *)&state);
+	}
+	else
+	{
+		pool_write(frontend, "Z", 1);
+	}
+	if (pool_flush(frontend))
+	{
+		return POOL_END;
+	}
+
+	*foundp = true;
+
+	if (pool_config->log_per_node_statement)
+		ereport(LOG,
+				(errmsg("fetch from memory cache"),
+				 errdetail("query result fetched from cache. statement: %s", contents)));
+
+	ereport(DEBUG1,
+			(errmsg("fetch from memory cache"),
+			 errdetail("query result found in the query cache, %s", contents)));
+
 	return POOL_CONTINUE;
 }
 
 /*
- * Simple and rough(thus unreliable) check if the query is likely
+ * Simple and rough (thus unreliable) check if the query is likely
  * SELECT. Just check if the query starts with SELECT or WITH. This
  * can be used before parse tree is available.
  */
@@ -682,10 +730,7 @@ bool pool_is_likely_select(char *query)
 		while (*query && isspace(*query))
 			query++;
 	}
-	if (! *query)
-	{
-		return false;
-	}
+	if (! *query) { return false; }
 
 	/*
 	 * Get rid of head comment.
@@ -701,10 +746,7 @@ bool pool_is_likely_select(char *query)
 			query++;
 			do_continue = true;
 		}
-		if (do_continue)
-		{
-			continue;
-		}
+		if (do_continue) { continue; }
 
 		while (*query && !strncmp(query, "\n", 2))
 		{
@@ -810,6 +852,21 @@ bool pool_is_allow_to_cache(Node *node, char *query)
 	if (pool_has_system_catalog(node))
 		return false;
 
+	/*
+	 * TABLESAMPLE is not allowed to cache.
+	 */
+	if (IsA(node, SelectStmt) && ((SelectStmt *)node)->fromClause)
+	{
+		List *tbl_list = ((SelectStmt *)node)->fromClause;
+		ListCell   *tbl;
+		foreach(tbl, tbl_list)
+		{
+			if (IsA(lfirst(tbl), RangeTableSample))
+				return false;
+		}
+	}
+
+
 	/*
 	 * If the table is in the while list, allow to cache even if it is
 	 * VIEW or unlogged table.
@@ -889,8 +946,8 @@ bool pool_is_table_in_white_list(const char *table_name)
  * Extract table oid from INSERT/UPDATE/DELETE/TRUNCATE/
  * DROP TABLE/ALTER TABLE/COPY FROM statement.
  * Returns number of oids.
- * In case of error, returns 0 (InvalidOid).
- * oids buffer (oidsp) will be discarded by subsequent call.
+ * In case of error, returns 0(InvalidOid).
+ * oids buffer(oidsp) will be discarded by subsequent call.
  */
 int pool_extract_table_oids(Node *node, int **oidsp)
 {
@@ -913,17 +970,17 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 	if (IsA(node, InsertStmt))
 	{
 		InsertStmt *stmt = (InsertStmt *)node;
-		table = nodeToString(stmt->relation);
+		table = make_table_name_from_rangevar(stmt->relation);
 	}
 	else if (IsA(node, UpdateStmt))
 	{
 		UpdateStmt *stmt = (UpdateStmt *)node;
-		table = get_relation_without_alias(stmt->relation);
+		table = make_table_name_from_rangevar(stmt->relation);
 	}
 	else if (IsA(node, DeleteStmt))
 	{
 		DeleteStmt *stmt = (DeleteStmt *)node;
-		table = get_relation_without_alias(stmt->relation);
+		table = make_table_name_from_rangevar(stmt->relation);
 	}
 
 #ifdef NOT_USED
@@ -935,14 +992,14 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 	else if (IsA(node, CreateStmt))
 	{
 		CreateStmt *stmt = (CreateStmt *)node;
-		table = nodeToString(stmt->relation);
+		table = make_table_name_from_rangevar(stmt->relation);
 	}
 #endif
 
 	else if (IsA(node, AlterTableStmt))
 	{
 		AlterTableStmt *stmt = (AlterTableStmt *)node;
-		table = nodeToString(stmt->relation);
+		table = make_table_name_from_rangevar(stmt->relation);
 	}
 
 	else if (IsA(node, CopyStmt))
@@ -950,7 +1007,7 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 		CopyStmt *stmt = (CopyStmt *)node;
 		if (stmt->is_from)		/* COPY FROM? */
 		{
-			table = nodeToString(stmt->relation);
+			table = make_table_name_from_rangevar(stmt->relation);
 		}
 		else
 		{
@@ -972,20 +1029,20 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 		/* Here, stmt->objects is list of target relation info.  The
 		 * first cell of target relation info is a list (possibly)
 		 * consists of database, schema and relation.  We need to call
-		 * makeRangeVarFromNameList() before passing to nodeToString.
-		 * Otherwise we get weird excessively decorated relation name
-		 * (''table_name'').
+		 * makeRangeVarFromNameList() before passing to
+		 * make_table_name_from_rangevar. Otherwise we get weird excessively
+		 * decorated relation name (''table_name'').
 		 */
 		foreach(cell, stmt->objects)
 		{
-			if (num_oids >= POOL_MAX_DML_OIDS)
+			if (num_oids > POOL_MAX_DML_OIDS)
 			{
 				ereport(LOG,
 						(errmsg("memcache: error while extracting table oids. too many oids:%d", num_oids)));
 				return 0;
 			}
 
-			table = nodeToString(makeRangeVarFromNameList(lfirst(cell)));
+			table = make_table_name_from_rangevar(makeRangeVarFromNameList(lfirst(cell)));
 			oid = pool_table_name_to_oid(table);
 			if (oid > 0)
 			{
@@ -1004,14 +1061,14 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 
 		foreach(cell, stmt->relations)
 		{
-			if (num_oids >= POOL_MAX_DML_OIDS)
+			if (num_oids > POOL_MAX_DML_OIDS)
 			{
 				ereport(LOG,
 						(errmsg("memcache: error while extracting table oids. too many oids:%d", num_oids)));
 				return 0;
 			}
 
-			table = nodeToString(lfirst(cell));
+			table = make_table_name_from_rangevar(lfirst(cell));
 			oid = pool_table_name_to_oid(table);
 			if (oid > 0)
 			{
@@ -1032,13 +1089,6 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 	oid = pool_table_name_to_oid(table);
 	if (oid > 0)
 	{
-		if (num_oids >= POOL_MAX_DML_OIDS)
-		{
-			ereport(LOG,
-					(errmsg("memcache: error while extracting table oids. too many oids:%d", num_oids)));
-			return 0;
-		}
-
 		oids[num_oids++] = pool_table_name_to_oid(table);
 		ereport(DEBUG1,
 				(errmsg("memcache: extracting table oids: table: \"%s\" oid:%d", table, oid)));
@@ -1046,41 +1096,6 @@ int pool_extract_table_oids(Node *node, int **oidsp)
 	return num_oids;
 }
 
-/*
- * Get relation name without alias.  NodeToString() calls
- * _outRangeVar(). Unfortunately _outRangeVar() returns table with
- * alias ("t1 AS foo") as a table name if table alias is used. This
- * function just trim down "AS..." part from the table name when table
- * alias is used.
- */
-static char *get_relation_without_alias(RangeVar *relation)
-{
-	char *table;
-	char *p;
-
-	if (!IsA(relation, RangeVar))
-	{
-		ereport(WARNING,
-			(errmsg("memcache: extracting table name without alias: invalid node, not a RangeVar"),
-				 errdetail("passed in node is of type:%d",relation->type)));
-		return "";
-	}
-	table = nodeToString(relation);
-	if (relation->alias)
-	{
-		p = strchr(table, ' ');
-		if (!p)
-		{
-			ereport(LOG,
-				(errmsg("memcache: extracting table name without alias: parsing error"),
-					 errdetail("cannot locate space in name:\"%s\"", table)));
-			return "";
-		}
-		*p = '\0';
-	}
-	return table;		
-}
-
 #define POOL_OIDBUF_SIZE 1024
 static int* oidbuf;
 static int oidbufp;
@@ -1107,7 +1122,7 @@ void pool_add_dml_table_oid(int oid)
 		 */
 		oldcxt = MemoryContextSwitchTo(TopMemoryContext);
 		tmp = repalloc(oidbuf, sizeof(int) * oidbuf_size);
-		oldcxt = MemoryContextSwitchTo(oldcxt);
+		MemoryContextSwitchTo(oldcxt);
 		if (tmp == NULL)
 			return;
 
@@ -1202,7 +1217,7 @@ static void pool_discard_dml_table_oid(void)
  * using same table oid will add to the same file. If the SELECT uses
  * multiple tables, multiple table_oid_file will be created. When
  * INSERT/UPDATE/DELETE is executed, corresponding caches must be
- * deleted (cache invalidation) (when DROP TABLE, ALTER TABLE is
+ * deleted(cache invalidation) (when DROP TABLE, ALTER TABLE is
  * executed, the caches must be deleted as well). When database is
  * dropped, all caches belonging to the database must be deleted.
  */
@@ -1277,7 +1292,7 @@ int pool_get_database_oid_from_dbname(char *dbname)
 }
 
 /*
- * Add cache id (shmem case) or hash key (memcached case) to table oid
+ * Add cache id(shmem case) or hash key(memcached case) to table oid
  * map file.  Caller must hold shmem lock before calling this function
  * to avoid file extension conflict among different pgpool child
  * process.
@@ -1478,7 +1493,7 @@ void pool_discard_oid_maps_by_db(int dboid)
 }
 
 /*
- * Read cache id (shmem case) or hash key (memcached case) from table
+ * Reading cache id(shmem case) or hash key(memcached case) from table
  * oid map file according to table_oids and discard cache entries.  If
  * unlink is true, the file will be unlinked after successful cache
  * removal.
@@ -1508,8 +1523,7 @@ static void pool_invalidate_query_cache(int num_table_oids, int *table_oid, bool
 	/*
 	 * Create memqcache_oiddir/database_oid
 	 */
-	if (dboid == 0)
-	{
+	if (dboid == 0) {
 		dboid = pool_get_database_oid();
 		ereport(DEBUG1,
 			(errmsg("memcache invalidating query cache"),
@@ -1642,10 +1656,9 @@ static void pool_invalidate_query_cache(int num_table_oids, int *table_oid, bool
 }
 
 /*
- * Reset SELECT data buffers.  If reset_dml_oids is true, call
- * pool_discard_dml_table_oid() to reset table oids used in DML statements.
+ * Reset SELECT data buffers
  */
-static void pool_reset_memqcache_buffer(bool reset_dml_oids)
+static void pool_reset_memqcache_buffer(void)
 {
 	POOL_SESSION_CONTEXT * session_context;
 
@@ -1685,10 +1698,7 @@ static void pool_reset_memqcache_buffer(bool reset_dml_oids)
 			session_context->query_context->temp_cache = NULL;
 		}
 	}
-
-	if (reset_dml_oids)
-		pool_discard_dml_table_oid();
-
+	pool_discard_dml_table_oid();
 	pool_tmp_stats_reset_num_selects();
 }
 
@@ -1783,11 +1793,7 @@ void
 pool_clear_memory_cache(void)
 {
 	size_t size;
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	POOL_SETMASK2(&BlockSig, &oldmask);
 	pool_shmem_lock();
@@ -3101,16 +3107,12 @@ static void pool_check_and_discard_cache_buffer(int num_oids, int *oids)
 void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *node, char state)
 {
 	POOL_SESSION_CONTEXT *session_context;
+	pool_sigset_t oldmask;
 	char *cache_buffer;
 	size_t len;
 	int num_oids;
 	int *oids;
 	int i;
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
 
 	session_context = pool_get_session_context(true);
 
@@ -3179,7 +3181,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 			pool_stats_count_up_num_selects(1);
 
 			/* Reset temp buffer */
-			pool_reset_memqcache_buffer(true);
+			pool_reset_memqcache_buffer();
 		}
 		else
 		{
@@ -3231,7 +3233,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 	else if (is_rollback_query(node))	/* Rollback? */
 	{
 		/* Discard buffered data */
-		pool_reset_memqcache_buffer(true);
+		pool_reset_memqcache_buffer();
 	}
 	else if (is_commit_query(node))		/* Commit? */
 	{
@@ -3283,7 +3285,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 		/* Count up number of SELECT stats */
 		pool_stats_count_up_num_selects(pool_tmp_stats_get_num_selects());
 
-		pool_reset_memqcache_buffer(true);
+		pool_reset_memqcache_buffer();
 	}
 	else		/* Non cache safe queries */
 	{
@@ -3294,7 +3296,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 			{
 				/* Count up SELECT stats */
 				pool_stats_count_up_num_selects(1);
-				pool_reset_memqcache_buffer(true);
+				pool_reset_memqcache_buffer();
 			}
 			else
 			{
@@ -3317,7 +3319,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 				pool_invalidate_query_cache(num_oids, oids, true, dboid);
 				pool_discard_oid_maps_by_db(dboid);
 				pool_shmem_unlock();
-				pool_reset_memqcache_buffer(true);
+				pool_reset_memqcache_buffer();
 
 				pfree(oids);
 				ereport(DEBUG2,
@@ -3346,7 +3348,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 					pool_invalidate_query_cache(num_oids, oids, true, 0);
 					pool_shmem_unlock();
 					POOL_SETMASK(&oldmask);
-					pool_reset_memqcache_buffer(true);
+					pool_reset_memqcache_buffer();
 				}
 				else
 				{
@@ -3357,16 +3359,7 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 					 * if DML/DDL modifies the TABLE which SELECT uses.
 					 */
 					pool_check_and_discard_cache_buffer(num_oids, oids);
-					pool_reset_memqcache_buffer(false);
-				}
-			}
-			else if (num_oids == 0)
-			{
-				/* 
-				 * It is also necessary to clear cache buffers in case of
-				 * no oid queries (like BEGIN, CHECKPOINT, VACUUM, etc) too.
-				 */
-				pool_reset_memqcache_buffer(true);
+				} 
 			}
 		}
 	}
@@ -3390,11 +3383,7 @@ int pool_init_memqcache_stats(void)
 POOL_QUERY_CACHE_STATS *pool_get_memqcache_stats(void)
 {
 	static POOL_QUERY_CACHE_STATS mystats;
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	memset(&mystats, 0, sizeof(POOL_QUERY_CACHE_STATS));
 
@@ -3426,11 +3415,7 @@ void pool_reset_memqcache_stats(void)
  */
 long long int pool_stats_count_up_num_selects(long long int num)
 {
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	POOL_SETMASK2(&BlockSig, &oldmask);
 	pool_semaphore_lock(QUERY_CACHE_STATS_SEM);
@@ -3481,11 +3466,7 @@ void pool_tmp_stats_reset_num_selects(void)
  */
 long long int pool_stats_count_up_num_cache_hits(void)
 {
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	POOL_SETMASK2(&BlockSig, &oldmask);
 	pool_semaphore_lock(QUERY_CACHE_STATS_SEM);
@@ -3974,3 +3955,70 @@ POOL_SHMEM_STATS *pool_get_shmem_storage_stats(void)
 
 	return &mystats;
 }
+
+/*
+ * Send flash message to backend and forward any response to frontend.
+ */
+void forward_pending_data(POOL_CONNECTION *frontend, POOL_CONNECTION *backend)
+{
+	char kind;
+	int len;
+	char *buf;
+
+	/* If are doing extended query and we are in streaming replication
+	 * mode, we need to retrieve any response from backend.
+	 */
+	if (!pool_is_pending_response())
+	{
+		ereport(DEBUG1,
+				(errmsg("forward_pending_data: no peding data")));
+		return;
+	}
+
+	/* Send flush messsage to backend to retrieve response of backend */
+	pool_write(backend, "H", 1);		
+	len = htonl(sizeof(len));
+	pool_write_and_flush(backend, &len, sizeof(len));
+
+	/*
+	 * Then forward any response from backend
+	 */
+	for(;;)
+	{
+		pool_read(backend, &kind, 1);
+		ereport(DEBUG1,
+				(errmsg("forward_pending_data: forwarding kind: '%c'", kind)));
+		pool_write(frontend, &kind, 1);
+
+		pool_read(backend, &len, sizeof(len));
+		ereport(DEBUG1,
+				(errmsg("forward_pending_data: forwarding len: '%d'", ntohl(len))));
+		pool_write(frontend, &len, sizeof(len));
+
+		len = ntohl(len);
+		if ((len - sizeof(len)) > 0)
+		{
+			len -= sizeof(len);
+			ereport(DEBUG1,
+					(errmsg("forward_pending_data: fowarding rest of packet. len:%d", len)));
+			buf = palloc(len);
+			pool_read(backend, buf, len);
+			pool_write(frontend, buf, len);
+		}
+
+		/* check if there's any pending data */
+		if (!pool_ssl_pending(backend) && pool_read_buffer_is_empty(backend))
+		{
+			pool_set_timeout(0);
+			if (pool_check_fd(backend) != 0)
+			{
+				ereport(DEBUG1,
+						(errmsg("forward_pending_data: select shows no pending data")));
+				pool_set_timeout(-1);
+				break;
+			}
+			pool_set_timeout(-1);
+		}
+	}
+	pool_flush(frontend);
+}
